'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

var _treeDataUtils = require('./tree-data-utils');

var keyFromTreeIndex = function keyFromTreeIndex(_ref) {
  var treeIndex = _ref.treeIndex;
  return treeIndex;
};
var keyFromKey = function keyFromKey(_ref2) {
  var node = _ref2.node;
  return node.key;
};

describe('getVisibleNodeCount', function () {
  it('should handle flat data', function () {
    expect((0, _treeDataUtils.getVisibleNodeCount)({
      treeData: [{}, {}]
    })).toEqual(2);
  });

  it('should handle hidden nested data', function () {
    expect((0, _treeDataUtils.getVisibleNodeCount)({
      treeData: [{
        children: [{
          children: [{}, {}]
        }, {
          children: [{}]
        }]
      }, {}]
    })).toEqual(2);
  });

  it('should handle functions', function () {
    expect((0, _treeDataUtils.getVisibleNodeCount)({
      treeData: [{
        expanded: true,
        children: [{
          expanded: true,
          children: [{
            expanded: true,
            children: function children() {
              return { key: 1 };
            }
          }, {}]
        }, {
          children: [{}]
        }]
      }, {}]
    })).toEqual(6);
  });

  it('should handle partially expanded nested data', function () {
    expect((0, _treeDataUtils.getVisibleNodeCount)({
      treeData: [{
        expanded: true,
        children: [{
          expanded: true,
          children: [{}, {}]
        }, {
          children: [{}]
        }]
      }, {}]
    })).toEqual(6);
  });

  it('should handle fully expanded nested data', function () {
    expect((0, _treeDataUtils.getVisibleNodeCount)({
      treeData: [{
        expanded: true,
        children: [{
          expanded: true,
          children: [{}, {}]
        }, {
          expanded: true,
          children: [{}]
        }]
      }, {}]
    })).toEqual(7);
  });
});

describe('getVisibleNodeInfoAtIndex', function () {
  it('should handle empty data', function () {
    expect((0, _treeDataUtils.getVisibleNodeInfoAtIndex)({
      treeData: [],
      index: 1,
      getNodeKey: keyFromTreeIndex
    })).toEqual(null);
    expect((0, _treeDataUtils.getVisibleNodeInfoAtIndex)({
      treeData: null,
      index: 1,
      getNodeKey: keyFromTreeIndex
    })).toEqual(null);
    expect((0, _treeDataUtils.getVisibleNodeInfoAtIndex)({
      treeData: undefined,
      index: 1,
      getNodeKey: keyFromTreeIndex
    })).toEqual(null);
  });

  it('should handle flat data', function () {
    expect((0, _treeDataUtils.getVisibleNodeInfoAtIndex)({
      treeData: [{ key: 0 }],
      index: 0,
      getNodeKey: keyFromTreeIndex
    }).node.key).toEqual(0);
    expect((0, _treeDataUtils.getVisibleNodeInfoAtIndex)({
      treeData: [{ key: 0 }, { key: 1 }],
      index: 1,
      getNodeKey: keyFromTreeIndex
    }).node.key).toEqual(1);
  });

  it('should handle hidden nested data', function () {
    var result = (0, _treeDataUtils.getVisibleNodeInfoAtIndex)({
      treeData: [{
        key: 0,
        children: [{
          key: 1,
          children: [{ key: 2 }, { key: 3 }]
        }, {
          key: 4,
          children: [{ key: 5 }]
        }]
      }, { key: 6 }],
      index: 1,
      getNodeKey: keyFromTreeIndex
    });

    expect(result.node.key).toEqual(6);
    expect(result.path).toEqual([1]);
    expect(result.lowerSiblingCounts).toEqual([0]);
  });

  it('should handle partially expanded nested data', function () {
    var result = (0, _treeDataUtils.getVisibleNodeInfoAtIndex)({
      treeData: [{
        expanded: true,
        key: 0,
        children: [{
          key: 1,
          children: [{ key: 2 }, { key: 3 }]
        }, {
          expanded: true,
          key: 4,
          children: [{ key: 5 }]
        }]
      }, { key: 6 }],
      index: 3,
      getNodeKey: keyFromKey
    });

    expect(result.node.key).toEqual(5);
    expect(result.path).toEqual([0, 4, 5]);
    expect(result.lowerSiblingCounts).toEqual([1, 0, 0]);
  });

  it('should handle fully expanded nested data', function () {
    var result = (0, _treeDataUtils.getVisibleNodeInfoAtIndex)({
      treeData: [{
        expanded: true,
        key: 0,
        children: [{
          expanded: true,
          key: 1,
          children: [{ key: 2 }, { key: 3 }]
        }, {
          expanded: true,
          key: 4,
          children: [{ key: 5 }]
        }]
      }, { key: 6 }],
      index: 5,
      getNodeKey: keyFromTreeIndex
    });

    expect(result.node.key).toEqual(5);
    expect(result.path).toEqual([0, 4, 5]);
    expect(result.lowerSiblingCounts).toEqual([1, 0, 0]);
  });

  it('should handle an index that is larger than the data', function () {
    expect((0, _treeDataUtils.getVisibleNodeInfoAtIndex)({
      treeData: [{
        expanded: true,
        key: 0,
        children: [{
          expanded: true,
          key: 1,
          children: [{ key: 2 }, { key: 3 }]
        }, {
          expanded: true,
          key: 4,
          children: [{ key: 5 }]
        }]
      }, { key: 6 }],
      index: 7,
      getNodeKey: keyFromTreeIndex
    })).toEqual(null);
  });
});

describe('getNodeAtPath', function () {
  it('should handle empty data', function () {
    expect((0, _treeDataUtils.getNodeAtPath)({
      treeData: [],
      path: 1,
      getNodeKey: keyFromTreeIndex
    })).toEqual(null);
    expect((0, _treeDataUtils.getNodeAtPath)({
      treeData: null,
      path: 1,
      getNodeKey: keyFromTreeIndex
    })).toEqual(null);
    expect((0, _treeDataUtils.getNodeAtPath)({
      treeData: undefined,
      path: 1,
      getNodeKey: keyFromTreeIndex
    })).toEqual(null);
  });

  it('should handle flat data', function () {
    expect((0, _treeDataUtils.getNodeAtPath)({
      treeData: [{ key: 0 }],
      path: [0],
      getNodeKey: keyFromTreeIndex
    }).node.key).toEqual(0);
    expect((0, _treeDataUtils.getNodeAtPath)({
      treeData: [{ key: 0 }, { key: 1 }],
      path: [1],
      getNodeKey: keyFromTreeIndex
    }).node.key).toEqual(1);
  });

  it('should handle hidden nested data', function () {
    var result = (0, _treeDataUtils.getNodeAtPath)({
      treeData: [{
        key: 0,
        children: [{
          key: 1,
          children: [{ key: 2 }, { key: 3 }]
        }, {
          key: 4,
          children: [{ key: 5 }]
        }]
      }, { key: 6 }],
      path: [1],
      getNodeKey: keyFromTreeIndex
    });

    expect(result.node.key).toEqual(6);
  });

  it('should handle partially expanded nested data', function () {
    var result = (0, _treeDataUtils.getNodeAtPath)({
      treeData: [{
        expanded: true,
        key: 0,
        children: [{
          key: 1,
          children: [{ key: 2 }, { key: 3 }]
        }, {
          expanded: true,
          key: 4,
          children: [{ key: 5 }]
        }]
      }, { key: 6 }],
      path: [0, 4, 5],
      getNodeKey: keyFromKey
    });

    expect(result.node.key).toEqual(5);
  });

  it('should handle fully expanded nested data', function () {
    var result = (0, _treeDataUtils.getNodeAtPath)({
      treeData: [{
        expanded: true,
        key: 0,
        children: [{
          expanded: true,
          key: 1,
          children: [{ key: 2 }, { key: 3 }]
        }, {
          expanded: true,
          key: 4,
          children: [{ key: 5 }]
        }]
      }, { key: 6 }],
      path: [0, 4, 5],
      getNodeKey: keyFromTreeIndex
    });

    expect(result.node.key).toEqual(5);
  });

  it('should handle an index that is larger than the data', function () {
    expect((0, _treeDataUtils.getNodeAtPath)({
      treeData: [{
        expanded: true,
        key: 0,
        children: [{
          expanded: true,
          key: 1,
          children: [{ key: 2 }, { key: 3 }]
        }, {
          expanded: true,
          key: 4,
          children: [{ key: 5 }]
        }]
      }, { key: 6 }],
      path: [7],
      getNodeKey: keyFromTreeIndex
    })).toEqual(null);
  });
});

describe('getFlatDataFromTree', function () {
  it('should handle empty data', function () {
    expect((0, _treeDataUtils.getFlatDataFromTree)({ treeData: [], getNodeKey: keyFromTreeIndex })).toEqual([]);
    expect((0, _treeDataUtils.getFlatDataFromTree)({ treeData: null, getNodeKey: keyFromTreeIndex })).toEqual([]);
    expect((0, _treeDataUtils.getFlatDataFromTree)({ treeData: undefined, getNodeKey: keyFromTreeIndex })).toEqual([]);
  });

  it('should handle flat data', function () {
    expect((0, _treeDataUtils.getFlatDataFromTree)({
      ignoreCollapsed: true,
      getNodeKey: keyFromTreeIndex,
      treeData: [{ key: 0 }]
    })).toEqual([{
      node: { key: 0 },
      parentNode: null,
      path: [0],
      lowerSiblingCounts: [0],
      treeIndex: 0
    }]);

    expect((0, _treeDataUtils.getFlatDataFromTree)({
      ignoreCollapsed: true,
      treeData: [{ key: 0 }, { key: 1 }],
      getNodeKey: keyFromTreeIndex
    })).toEqual([{
      node: { key: 0 },
      parentNode: null,
      path: [0],
      lowerSiblingCounts: [1],
      treeIndex: 0
    }, {
      node: { key: 1 },
      parentNode: null,
      path: [1],
      lowerSiblingCounts: [0],
      treeIndex: 1
    }]);
  });

  it('should handle hidden nested data', function () {
    var treeData = [{
      key: 0,
      children: [{
        key: 1,
        children: [{ key: 2 }, { key: 3 }]
      }, {
        key: 4,
        children: [{ key: 5 }]
      }]
    }, { key: 6 }];

    expect((0, _treeDataUtils.getFlatDataFromTree)({
      ignoreCollapsed: true,
      getNodeKey: keyFromTreeIndex,
      treeData: treeData
    })).toEqual([{
      node: treeData[0],
      parentNode: null,
      path: [0],
      lowerSiblingCounts: [1],
      treeIndex: 0
    }, {
      node: treeData[1],
      parentNode: null,
      path: [1],
      lowerSiblingCounts: [0],
      treeIndex: 1
    }]);
  });

  it('should handle partially expanded nested data', function () {
    var treeData = [{
      expanded: true,
      key: 0,
      children: [{
        key: 1,
        children: [{ key: 2 }, { key: 3 }]
      }, {
        expanded: true,
        key: 4,
        children: [{ key: 5 }]
      }]
    }, { key: 6 }];

    expect((0, _treeDataUtils.getFlatDataFromTree)({
      ignoreCollapsed: true,
      getNodeKey: keyFromKey,
      treeData: treeData
    })).toEqual([{
      node: treeData[0],
      parentNode: null,
      path: [0],
      lowerSiblingCounts: [1],
      treeIndex: 0
    }, {
      node: treeData[0].children[0],
      parentNode: treeData[0],
      path: [0, 1],
      lowerSiblingCounts: [1, 1],
      treeIndex: 1
    }, {
      node: treeData[0].children[1],
      parentNode: treeData[0],
      path: [0, 4],
      lowerSiblingCounts: [1, 0],
      treeIndex: 2
    }, {
      node: treeData[0].children[1].children[0],
      parentNode: treeData[0].children[1],
      path: [0, 4, 5],
      lowerSiblingCounts: [1, 0, 0],
      treeIndex: 3
    }, {
      node: treeData[1],
      parentNode: null,
      path: [6],
      lowerSiblingCounts: [0],
      treeIndex: 4
    }]);
  });

  it('should handle fully expanded nested data', function () {
    var treeData = [{
      expanded: true,
      key: 0,
      children: [{
        expanded: true,
        key: 1,
        children: [{ key: 2 }, { key: 3 }]
      }, {
        expanded: true,
        key: 4,
        children: [{ key: 5 }]
      }]
    }, { key: 6 }];

    expect((0, _treeDataUtils.getFlatDataFromTree)({
      ignoreCollapsed: true,
      treeData: treeData,
      getNodeKey: keyFromTreeIndex
    })).toEqual([{
      node: treeData[0],
      parentNode: null,
      path: [0],
      lowerSiblingCounts: [1],
      treeIndex: 0
    }, {
      node: treeData[0].children[0],
      parentNode: treeData[0],
      path: [0, 1],
      lowerSiblingCounts: [1, 1],
      treeIndex: 1
    }, {
      node: treeData[0].children[0].children[0],
      parentNode: treeData[0].children[0],
      path: [0, 1, 2],
      lowerSiblingCounts: [1, 1, 1],
      treeIndex: 2
    }, {
      node: treeData[0].children[0].children[1],
      parentNode: treeData[0].children[0],
      path: [0, 1, 3],
      lowerSiblingCounts: [1, 1, 0],
      treeIndex: 3
    }, {
      node: treeData[0].children[1],
      parentNode: treeData[0],
      path: [0, 4],
      lowerSiblingCounts: [1, 0],
      treeIndex: 4
    }, {
      node: treeData[0].children[1].children[0],
      parentNode: treeData[0].children[1],
      path: [0, 4, 5],
      lowerSiblingCounts: [1, 0, 0],
      treeIndex: 5
    }, {
      node: treeData[1],
      parentNode: null,
      path: [6],
      lowerSiblingCounts: [0],
      treeIndex: 6
    }]);
  });
});

describe('changeNodeAtPath', function () {
  it('should handle empty data', function () {
    var noChildrenError = new Error('Path referenced children of node with no children.');
    var noNodeError = new Error('No node found at the given path.');
    expect(function () {
      return (0, _treeDataUtils.changeNodeAtPath)({
        treeData: [],
        path: [1],
        newNode: {},
        getNodeKey: keyFromTreeIndex
      });
    }).toThrow(noNodeError);
    expect(function () {
      return (0, _treeDataUtils.changeNodeAtPath)({
        treeData: null,
        path: [1],
        newNode: {},
        getNodeKey: keyFromTreeIndex
      });
    }).toThrow(noChildrenError);
    expect(function () {
      return (0, _treeDataUtils.changeNodeAtPath)({
        treeData: null,
        path: [1, 2],
        newNode: {},
        getNodeKey: keyFromTreeIndex
      });
    }).toThrow(noChildrenError);
    expect(function () {
      return (0, _treeDataUtils.changeNodeAtPath)({
        treeData: undefined,
        path: [1],
        newNode: {},
        getNodeKey: keyFromTreeIndex
      });
    }).toThrow(noChildrenError);
  });

  it('should handle flat data', function () {
    expect((0, _treeDataUtils.changeNodeAtPath)({
      treeData: [{ key: 0 }],
      path: [0],
      newNode: { key: 1 },
      getNodeKey: keyFromKey
    })).toEqual([{ key: 1 }]);

    expect((0, _treeDataUtils.changeNodeAtPath)({
      treeData: [{ key: 0 }, { key: 'a' }],
      path: ['a'],
      newNode: { key: 1 },
      getNodeKey: keyFromKey
    })).toEqual([{ key: 0 }, { key: 1 }]);
  });

  it('should handle nested data', function () {
    var result = (0, _treeDataUtils.changeNodeAtPath)({
      treeData: [{
        expanded: true,
        key: 0,
        children: [{
          key: 'b',
          children: [{ key: 2 }, { key: 3 }, { key: 'f' }]
        }, {
          expanded: true,
          key: 'r',
          children: [{ key: 5 }, { key: 8 }, { key: 7 }]
        }]
      }, { key: 6 }],
      path: [0, 2, 5],
      newNode: { food: 'pancake' },
      getNodeKey: keyFromTreeIndex
    });

    expect(result[0].children[1].children[2].food).toEqual('pancake');
  });

  it('should handle adding children', function () {
    var result = (0, _treeDataUtils.changeNodeAtPath)({
      treeData: [{
        expanded: true,
        key: 0,
        children: [{
          key: 'b',
          children: [{ key: 2 }, { key: 3 }, { key: 'f' }]
        }, {
          expanded: true,
          key: 'r',
          children: [{ key: 5 }, { key: 8 }, { key: 7 }]
        }]
      }, { key: 6 }],
      path: [0, 2, 5],
      newNode: function newNode(_ref3) {
        var node = _ref3.node;
        return _extends({}, node, {
          children: [{ food: 'pancake' }]
        });
      },
      getNodeKey: keyFromTreeIndex
    });

    expect(result[0].children[1].children[2].children[0].food).toEqual('pancake');
  });

  it('should handle adding children to the root', function () {
    expect((0, _treeDataUtils.changeNodeAtPath)({
      treeData: [],
      path: [],
      newNode: function newNode(_ref4) {
        var node = _ref4.node;
        return _extends({}, node, {
          children: [].concat(_toConsumableArray(node.children), [{ key: 1 }])
        });
      },
      getNodeKey: keyFromKey
    })).toEqual([{ key: 1 }]);

    expect((0, _treeDataUtils.changeNodeAtPath)({
      treeData: [{ key: 0 }],
      path: [],
      newNode: function newNode(_ref5) {
        var node = _ref5.node;
        return _extends({}, node, {
          children: [].concat(_toConsumableArray(node.children), [{ key: 1 }])
        });
      },
      getNodeKey: keyFromKey
    })).toEqual([{ key: 0 }, { key: 1 }]);
  });

  it('should delete data when falsey node passed', function () {
    var result = (0, _treeDataUtils.changeNodeAtPath)({
      treeData: [{
        expanded: true,
        key: 'b',
        children: [{ key: 'f' }]
      }, {
        expanded: true,
        key: 'r',
        children: [{ key: 7 }]
      }, { key: 6 }],
      path: [2, 3],
      newNode: null,
      getNodeKey: keyFromTreeIndex
    });

    expect(result[1].children.length).toEqual(0);
  });

  it('should delete data on the top level', function () {
    var treeData = [{
      expanded: true,
      key: 'b',
      children: [{ key: 'f' }]
    }, {
      expanded: true,
      key: 'r',
      children: [{ key: 7 }]
    }, { key: 6 }];
    var result = (0, _treeDataUtils.changeNodeAtPath)({
      treeData: treeData,
      path: [2],
      newNode: null,
      getNodeKey: keyFromTreeIndex
    });

    expect(result).toEqual([treeData[0], treeData[2]]);
  });

  it('should handle a path that is too long', function () {
    var treeData = [{
      expanded: true,
      key: 0,
      children: [{
        expanded: true,
        key: 1,
        children: [{ key: 2 }, { key: 3 }]
      }]
    }];

    expect(function () {
      return (0, _treeDataUtils.changeNodeAtPath)({
        treeData: treeData,
        path: [0, 1, 2, 4],
        newNode: { a: 1 },
        getNodeKey: keyFromKey
      });
    }).toThrow(new Error('Path referenced children of node with no children.'));
  });

  it('should handle a path that does not exist', function () {
    var treeData = [{
      expanded: true,
      key: 0,
      children: [{
        expanded: true,
        key: 1,
        children: [{ key: 2 }, { key: 3 }]
      }]
    }];

    expect(function () {
      return (0, _treeDataUtils.changeNodeAtPath)({
        treeData: treeData,
        path: [0, 2],
        newNode: { a: 1 },
        getNodeKey: keyFromKey
      });
    }).toThrowError('No node found at the given path.');
  });
});

describe('addNodeUnderParent', function () {
  it('should handle empty data', function () {
    expect((0, _treeDataUtils.addNodeUnderParent)({
      treeData: [],
      parentKey: null,
      newNode: {},
      getNodeKey: keyFromTreeIndex
    })).toEqual({ treeData: [{}], treeIndex: 0 });
    expect((0, _treeDataUtils.addNodeUnderParent)({
      treeData: null,
      parentKey: null,
      newNode: {},
      getNodeKey: keyFromTreeIndex
    })).toEqual({ treeData: [{}], treeIndex: 0 });
    expect((0, _treeDataUtils.addNodeUnderParent)({
      treeData: undefined,
      parentKey: null,
      newNode: {},
      getNodeKey: keyFromTreeIndex
    })).toEqual({ treeData: [{}], treeIndex: 0 });
  });

  it('should handle a parentPath that does not exist', function () {
    var treeData = [{
      expanded: true,
      key: 0,
      children: [{
        expanded: true,
        key: 1,
        children: [{ key: 2 }, { key: 3 }]
      }]
    }];

    expect(function () {
      return (0, _treeDataUtils.addNodeUnderParent)({
        treeData: treeData,
        parentKey: 'fake',
        newNode: { a: 1 },
        getNodeKey: keyFromKey
      });
    }).toThrowError('No node found with the given key.');
  });

  it('should handle flat data', function () {
    // Older sibling of only node
    expect((0, _treeDataUtils.addNodeUnderParent)({
      treeData: [{ key: 0 }],
      parentKey: null,
      newNode: { key: 1 },
      getNodeKey: keyFromTreeIndex
    })).toEqual({ treeData: [{ key: 0 }, { key: 1 }], treeIndex: 1 });

    // Child of only node
    expect((0, _treeDataUtils.addNodeUnderParent)({
      treeData: [{ key: 0 }],
      parentKey: 0,
      newNode: { key: 1 },
      getNodeKey: keyFromTreeIndex
    })).toEqual({ treeData: [{ key: 0, children: [{ key: 1 }] }], treeIndex: 1 });

    expect((0, _treeDataUtils.addNodeUnderParent)({
      treeData: [{ key: 0 }, { key: 'a' }],
      parentKey: 'a',
      newNode: { key: 1 },
      getNodeKey: keyFromKey
    })).toEqual({
      treeData: [{ key: 0 }, { key: 'a', children: [{ key: 1 }] }],
      treeIndex: 2
    });
  });

  // Tree looks like this
  //      /\
  //     0  6
  //    / \
  //   1   5
  //  / \
  // 2   3
  //      \
  //       4
  var nestedParams = {
    treeData: [{
      expanded: true,
      key: 0,
      children: [{
        expanded: true,
        key: 1,
        children: [{ key: 2 }, {
          expanded: false,
          key: 3,
          children: [{ key: 4 }]
        }]
      }, { key: 5 }]
    }, { key: 6 }],
    newNode: { key: 'new' }
  };

  it('should handle nested data #1', function () {
    var result = (0, _treeDataUtils.addNodeUnderParent)(_extends({}, nestedParams, {
      parentKey: 0,
      getNodeKey: keyFromKey
    }));

    expect(result.treeData[0].children[2]).toEqual(nestedParams.newNode);
    expect(result.treeIndex).toEqual(5);
  });

  it('should handle nested data #2', function () {
    var result = (0, _treeDataUtils.addNodeUnderParent)(_extends({}, nestedParams, {
      parentKey: 1,
      getNodeKey: keyFromKey
    }));

    expect(result.treeData[0].children[0].children[2]).toEqual(nestedParams.newNode);
    expect(result.treeIndex).toEqual(4);
  });

  it('should handle nested data #3', function () {
    var result = (0, _treeDataUtils.addNodeUnderParent)(_extends({}, nestedParams, {
      parentKey: 3,
      getNodeKey: keyFromKey
    }));

    expect(result.treeData[0].children[0].children[1].children[1]).toEqual(nestedParams.newNode);
    expect(result.treeIndex).toEqual(5);
  });

  it('should handle nested data #1 (using tree index as key)', function () {
    var result = (0, _treeDataUtils.addNodeUnderParent)(_extends({}, nestedParams, {
      parentKey: 0,
      getNodeKey: keyFromTreeIndex
    }));

    expect(result.treeData[0].children[2]).toEqual(nestedParams.newNode);
    expect(result.treeIndex).toEqual(5);
  });

  it('should handle nested data #2 (using tree index as key)', function () {
    var result = (0, _treeDataUtils.addNodeUnderParent)(_extends({}, nestedParams, {
      parentKey: 1,
      getNodeKey: keyFromTreeIndex
    }));

    expect(result.treeData[0].children[0].children[2]).toEqual(nestedParams.newNode);
    expect(result.treeIndex).toEqual(4);
  });

  it('should handle nested data #3 (using tree index as key)', function () {
    var result = (0, _treeDataUtils.addNodeUnderParent)(_extends({}, nestedParams, {
      parentKey: 3,
      getNodeKey: keyFromTreeIndex
    }));

    expect(result.treeData[0].children[0].children[1].children[1]).toEqual(nestedParams.newNode);
    expect(result.treeIndex).toEqual(5);
  });
});

describe('insertNode', function () {
  it('should handle empty data', function () {
    expect((0, _treeDataUtils.insertNode)({
      treeData: [],
      depth: 0,
      minimumTreeIndex: 0,
      newNode: {},
      getNodeKey: keyFromTreeIndex
    })).toEqual({ parentNode: null, treeData: [{}], treeIndex: 0, path: [0] });
    expect((0, _treeDataUtils.insertNode)({
      treeData: null,
      depth: 0,
      minimumTreeIndex: 0,
      newNode: {},
      getNodeKey: keyFromTreeIndex
    })).toEqual({ parentNode: null, treeData: [{}], treeIndex: 0, path: [0] });
    expect((0, _treeDataUtils.insertNode)({
      treeData: undefined,
      depth: 0,
      minimumTreeIndex: 0,
      newNode: {},
      getNodeKey: keyFromTreeIndex
    })).toEqual({ parentNode: null, treeData: [{}], treeIndex: 0, path: [0] });
  });

  it('should handle a depth that is deeper than any branch in the tree', function () {
    var treeData = [{
      expanded: true,
      key: 0,
      children: [{
        expanded: true,
        key: 1,
        children: [{ key: 2 }, { key: 3 }]
      }]
    }];

    expect((0, _treeDataUtils.insertNode)({
      treeData: treeData,
      depth: 4,
      minimumTreeIndex: 0,
      newNode: { key: 'new' },
      getNodeKey: keyFromKey
    }).treeData[0]).toEqual({ key: 'new' });
  });

  it('should handle a minimumTreeIndex that is too big', function () {
    var treeData = [{
      expanded: true,
      key: 0,
      children: [{
        expanded: true,
        key: 1,
        children: [{ key: 2 }, { key: 3 }]
      }]
    }, { key: 4 }];

    var insertResult = (0, _treeDataUtils.insertNode)({
      treeData: treeData,
      depth: 0,
      minimumTreeIndex: 15,
      newNode: { key: 'new' },
      getNodeKey: keyFromKey
    });
    expect(insertResult.treeData[2]).toEqual({ key: 'new' });
    expect(insertResult.treeIndex).toEqual(5);
    expect(insertResult.path).toEqual(['new']);

    insertResult = (0, _treeDataUtils.insertNode)({
      treeData: treeData,
      depth: 2,
      minimumTreeIndex: 15,
      newNode: { key: 'new' },
      getNodeKey: keyFromKey
    });

    expect(insertResult.treeData[1].children[0]).toEqual({ key: 'new' });
    expect(insertResult.treeIndex).toEqual(5);
    expect(insertResult.path).toEqual([4, 'new']);
  });

  it('should handle flat data (before)', function () {
    expect((0, _treeDataUtils.insertNode)({
      treeData: [{ key: 0 }],
      depth: 0,
      minimumTreeIndex: 0,
      newNode: { key: 1 },
      getNodeKey: keyFromKey
    })).toEqual({
      parentNode: null,
      treeData: [{ key: 1 }, { key: 0 }],
      treeIndex: 0,
      path: [1]
    });
  });

  it('should handle flat data (after)', function () {
    expect((0, _treeDataUtils.insertNode)({
      treeData: [{ key: 0 }],
      depth: 0,
      minimumTreeIndex: 1,
      newNode: { key: 1 },
      getNodeKey: keyFromKey
    })).toEqual({
      parentNode: null,
      treeData: [{ key: 0 }, { key: 1 }],
      treeIndex: 1,
      path: [1]
    });
  });

  it('should handle flat data (child)', function () {
    expect((0, _treeDataUtils.insertNode)({
      treeData: [{ key: 0 }],
      depth: 1,
      minimumTreeIndex: 1,
      newNode: { key: 1 },
      getNodeKey: keyFromKey
    })).toEqual({
      parentNode: { key: 0, children: [{ key: 1 }] },
      treeData: [{ key: 0, children: [{ key: 1 }] }],
      treeIndex: 1,
      path: [0, 1]
    });
  });

  // Tree looks like this
  //      /\
  //     0  6
  //    / \
  //   1   5
  //  / \
  // 2   3
  //      \
  //       4
  var nestedParams = {
    treeData: [
    // Depth 0
    {
      expanded: true,
      key: 0,
      children: [
      // Depth 1
      {
        expanded: true,
        key: 1,
        children: [
        // Depth 2
        { key: 2 }, {
          expanded: false,
          key: 3,
          children: [
          // Depth 3
          { key: 4 }]
        }, { key: 5 }]
      }, { key: 6 }]
    }, { key: 7 }],
    newNode: { key: 'new' },
    getNodeKey: keyFromKey
  };

  it('should handle nested data #1', function () {
    var result = (0, _treeDataUtils.insertNode)(_extends({}, nestedParams, {
      depth: 1,
      minimumTreeIndex: 4
    }));

    expect(result.treeData[0].children[1]).toEqual(nestedParams.newNode);
    expect(result.treeIndex).toEqual(5);
    expect(result.path).toEqual([0, 'new']);
  });

  it('should handle nested data #2', function () {
    var result = (0, _treeDataUtils.insertNode)(_extends({}, nestedParams, {
      depth: 2,
      ignoreCollapsed: true,
      minimumTreeIndex: 5
    }));

    expect(result.treeData[0].children[0].children[3]).toEqual(nestedParams.newNode);
    expect(result.treeIndex).toEqual(5);
    expect(result.path).toEqual([0, 1, 'new']);

    result = (0, _treeDataUtils.insertNode)(_extends({}, nestedParams, {
      depth: 2,
      ignoreCollapsed: false,
      minimumTreeIndex: 5
    }));

    expect(result.treeData[0].children[0].children[2]).toEqual(nestedParams.newNode);
    expect(result.treeIndex).toEqual(5);
    expect(result.path).toEqual([0, 1, 'new']);
  });

  it('should handle nested data #3', function () {
    var result = (0, _treeDataUtils.insertNode)(_extends({}, nestedParams, {
      depth: 3,
      minimumTreeIndex: 3
    }));

    expect(result.treeData[0].children[0].children[0].children[0]).toEqual(nestedParams.newNode);
    expect(result.treeIndex).toEqual(3);
    expect(result.path).toEqual([0, 1, 2, 'new']);
  });

  it('should handle nested data #4', function () {
    expect((0, _treeDataUtils.insertNode)({
      treeData: [{ key: 0, expanded: true, children: [{ key: 1 }] }, { key: 2 }],
      newNode: { key: 'new' },
      depth: 1,
      minimumTreeIndex: 3,
      getNodeKey: keyFromTreeIndex
    })).toEqual({
      parentNode: { key: 2, children: [{ key: 'new' }] },
      treeData: [{ key: 0, expanded: true, children: [{ key: 1 }] }, { key: 2, children: [{ key: 'new' }] }],
      treeIndex: 3,
      path: [2, 3]
    });
  });

  it('should work with a preceding node with children #1', function () {
    expect((0, _treeDataUtils.insertNode)({
      treeData: [{ children: [{}] }, { expanded: true, children: [{}, {}] }],
      newNode: { key: 'new' },
      depth: 1,
      minimumTreeIndex: 3,
      getNodeKey: keyFromTreeIndex
    })).toEqual({
      parentNode: { expanded: true, children: [{}, { key: 'new' }, {}] },
      treeData: [{ children: [{}] }, { expanded: true, children: [{}, { key: 'new' }, {}] }],
      treeIndex: 3,
      path: [1, 3]
    });
  });

  it('should work with a preceding node with children #2', function () {
    expect((0, _treeDataUtils.insertNode)({
      treeData: [{ children: [{}] }, { expanded: true, children: [{}, {}] }],
      newNode: { key: 'new' },
      depth: 2,
      minimumTreeIndex: 4,
      getNodeKey: keyFromTreeIndex
    })).toEqual({
      parentNode: { children: [{ key: 'new' }] },
      treeData: [{ children: [{}] }, { expanded: true, children: [{}, { children: [{ key: 'new' }] }] }],
      treeIndex: 4,
      path: [1, 3, 4]
    });
  });

  it('should work with a preceding node with children #3', function () {
    expect((0, _treeDataUtils.insertNode)({
      treeData: [{ children: [{}, {}, {}, {}] }, { expanded: true, children: [{}, {}] }],
      newNode: { key: 'new' },
      depth: 2,
      minimumTreeIndex: 4,
      getNodeKey: keyFromTreeIndex
    })).toEqual({
      parentNode: { children: [{ key: 'new' }] },
      treeData: [{ children: [{}, {}, {}, {}] }, { expanded: true, children: [{}, { children: [{ key: 'new' }] }] }],
      treeIndex: 4,
      path: [1, 3, 4]
    });
  });

  it('should work with nodes with an empty children array', function () {
    expect((0, _treeDataUtils.insertNode)({
      treeData: [{
        expanded: true,
        children: [{
          expanded: true,
          children: [{ children: [] }]
        }]
      }],
      newNode: { key: 'new' },
      depth: 2,
      minimumTreeIndex: 2,
      getNodeKey: keyFromTreeIndex
    })).toEqual({
      parentNode: {
        expanded: true,
        children: [{ key: 'new' }, { children: [] }]
      },
      treeData: [{
        expanded: true,
        children: [{
          expanded: true,
          children: [{ key: 'new' }, { children: [] }]
        }]
      }],
      treeIndex: 2,
      path: [0, 1, 2]
    });
  });
});

describe('walk', function () {
  it('should handle empty data', function () {
    [[], null, undefined].forEach(function (treeData) {
      expect(function () {
        return (0, _treeDataUtils.walk)({
          treeData: treeData,
          getNodeKey: keyFromTreeIndex,
          callback: function callback() {
            throw new Error('callback ran');
          }
        });
      }).not.toThrow();
    });
  });

  it('should handle flat and nested data', function () {
    [{
      treeData: [{}],
      expected: 1
    }, {
      treeData: [{}, {}],
      expected: 2
    }, {
      treeData: [{}, { children: [{}] }, {}],
      expected: 3
    }, {
      treeData: [{}, { children: [{}] }, {}],
      ignoreCollapsed: false,
      expected: 4
    }].forEach(function (_ref6) {
      var treeData = _ref6.treeData;
      var expected = _ref6.expected;
      var _ref6$ignoreCollapsed = _ref6.ignoreCollapsed;
      var ignoreCollapsed = _ref6$ignoreCollapsed === undefined ? true : _ref6$ignoreCollapsed;

      var callCount = 0;
      (0, _treeDataUtils.walk)({
        treeData: treeData,
        ignoreCollapsed: ignoreCollapsed,
        getNodeKey: keyFromTreeIndex,
        callback: function callback() {
          callCount += 1;
        }
      });

      expect(callCount).toEqual(expected);
    });
  });

  it('should return correct params', function () {
    var paths = [[0], [1], [1, 2], [3]];
    var counter = 0;

    (0, _treeDataUtils.walk)({
      treeData: [{}, { children: [{}] }, {}],
      ignoreCollapsed: false,
      getNodeKey: keyFromTreeIndex,
      callback: function callback(_ref7) {
        var treeIndex = _ref7.treeIndex;
        var path = _ref7.path;

        expect(treeIndex).toEqual(counter);
        expect(path).toEqual(paths[treeIndex]);
        counter += 1;
      }
    });
  });

  it('should cut walk short when false is returned', function () {
    var treeData = [{
      expanded: true,
      key: 0,
      children: [{ key: 2 }, { key: 3 }]
    }, { key: 6 }];

    expect(function () {
      return (0, _treeDataUtils.walk)({
        treeData: treeData,
        getNodeKey: keyFromTreeIndex,
        callback: function callback(_ref8) {
          var node = _ref8.node;

          if (node.key === 2) {
            // Cut walk short with false
            return false;
          } else if (node.key === 3) {
            throw new Error('walk not terminated by false');
          }

          return true;
        }
      });
    }).not.toThrow();
  });

  it('can get parents while walking', function () {
    var treeData = [{ key: 1, children: [{ key: 12, children: [{ key: 3 }] }, { key: 4 }] }, { key: 5 }];
    var results = [];
    (0, _treeDataUtils.walk)({
      treeData: treeData,
      getNodeKey: keyFromTreeIndex,
      ignoreCollapsed: false,
      callback: function callback(_ref9) {
        var parentNode = _ref9.parentNode;

        results.push(parentNode ? parentNode.key : null);
      }
    });

    expect(results).toEqual([null, 1, 12, 1, null]);
  });
});

describe('getTreeFromFlatData', function () {
  var rootKey = -1;
  var argDefaults = {
    rootKey: rootKey,
    getKey: function getKey(node) {
      return node.key;
    },
    getParentKey: function getParentKey(node) {
      return node.parentKey;
    }
  };

  var checkFunction = function checkFunction(_ref10) {
    var flatData = _ref10.flatData;
    var expected = _ref10.expected;

    expect((0, _treeDataUtils.getTreeFromFlatData)(_extends({}, argDefaults, {
      flatData: flatData
    }))).toEqual(expected);
  };

  it('should handle empty data', function () {
    [{ flatData: [], expected: [] }, { flatData: null, expected: [] }, { flatData: undefined, expected: [] }].forEach(checkFunction);
  });

  it('should handle [depth == 1] data', function () {
    [{
      flatData: [{ key: 1, parentKey: rootKey }, { key: 2, parentKey: rootKey }],
      expected: [{ key: 1, parentKey: rootKey }, { key: 2, parentKey: rootKey }]
    }, {
      flatData: [{ key: '1', parentKey: rootKey }, { key: '2', parentKey: rootKey }],
      expected: [{ key: '1', parentKey: rootKey }, { key: '2', parentKey: rootKey }]
    }].forEach(checkFunction);
  });

  it('should handle [depth == 2] data', function () {
    [{
      flatData: [{ key: 1, parentKey: rootKey }, { key: 2, parentKey: 1 }],
      expected: [{
        key: 1,
        parentKey: rootKey,
        children: [{ key: 2, parentKey: 1 }]
      }]
    }, {
      flatData: [{ key: '1', parentKey: rootKey }, { key: '2', parentKey: '1' }],
      expected: [{
        key: '1',
        parentKey: rootKey,
        children: [{ key: '2', parentKey: '1' }]
      }]
    }].forEach(checkFunction);
  });

  it('should handle [depth > 2] nested data', function () {
    [{
      flatData: [{ key: 3, parentKey: 2 }, { key: 1, parentKey: rootKey }, { key: 2, parentKey: 1 }],
      expected: [{
        key: 1,
        parentKey: rootKey,
        children: [{
          key: 2,
          parentKey: 1,
          children: [{ key: 3, parentKey: 2 }]
        }]
      }]
    }, {
      flatData: [{ key: 4, parentKey: 2 }, { key: 3, parentKey: 2 }, { key: 7, parentKey: rootKey }, { key: 1, parentKey: rootKey }, { key: 2, parentKey: 1 }, { key: 6, parentKey: 1 }],
      expected: [{ key: 7, parentKey: rootKey }, {
        key: 1,
        parentKey: rootKey,
        children: [{
          key: 2,
          parentKey: 1,
          children: [{ key: 4, parentKey: 2 }, { key: 3, parentKey: 2 }]
        }, { key: 6, parentKey: 1 }]
      }]
    }].forEach(checkFunction);
  });
});

describe('map', function () {
  var checkFunction = function checkFunction(_ref11) {
    var treeData = _ref11.treeData;
    var getNodeKey = _ref11.getNodeKey;
    var callback = _ref11.callback;
    var ignoreCollapsed = _ref11.ignoreCollapsed;
    var expected = _ref11.expected;

    expect((0, _treeDataUtils.map)({
      treeData: treeData,
      getNodeKey: getNodeKey,
      callback: callback,
      ignoreCollapsed: ignoreCollapsed
    })).toEqual(expected);
  };

  it('should handle empty data', function () {
    [{
      treeData: [],
      getNodeKey: keyFromKey,
      callback: function callback(_ref12) {
        var node = _ref12.node;
        return node;
      },
      expected: []
    }, {
      treeData: null,
      getNodeKey: keyFromKey,
      callback: function callback(_ref13) {
        var node = _ref13.node;
        return node;
      },
      expected: []
    }, {
      treeData: undefined,
      getNodeKey: keyFromKey,
      callback: function callback(_ref14) {
        var node = _ref14.node;
        return node;
      },
      expected: []
    }].forEach(checkFunction);
  });

  it('can return tree as-is', function () {
    [{
      getNodeKey: keyFromKey,
      callback: function callback(_ref15) {
        var node = _ref15.node;
        return node;
      },
      treeData: [{ key: 1 }, { key: 2 }],
      expected: [{ key: 1 }, { key: 2 }]
    }, {
      getNodeKey: keyFromKey,
      callback: function callback(_ref16) {
        var node = _ref16.node;
        return node;
      },
      treeData: [{ key: 1, children: [{ key: 2 }] }],
      expected: [{ key: 1, children: [{ key: 2 }] }]
    }, {
      getNodeKey: keyFromKey,
      callback: function callback(_ref17) {
        var node = _ref17.node;
        return node;
      },
      treeData: [{
        key: 1,
        children: [{ key: 12, children: [{ key: 3 }] }, { key: 4 }]
      }, { key: 5 }],
      expected: [{
        key: 1,
        children: [{ key: 12, children: [{ key: 3 }] }, { key: 4 }]
      }, { key: 5 }]
    }].forEach(checkFunction);
  });

  it('can truncate part of the tree', function () {
    [{
      getNodeKey: keyFromKey,
      callback: function callback(_ref18) {
        var node = _ref18.node;
        return node.key === 1 ? _extends({}, node, { children: [] }) : node;
      },
      treeData: [{
        key: 1,
        children: [{ key: 12, children: [{ key: 3 }] }, { key: 4 }]
      }, { key: 5 }],
      expected: [{ key: 1, children: [] }, { key: 5 }]
    }].forEach(checkFunction);
  });

  it('can get parents', function () {
    checkFunction({
      getNodeKey: keyFromKey,
      callback: function callback(_ref19) {
        var node = _ref19.node;
        var parentNode = _ref19.parentNode;
        return _extends({}, node, {
          parentKey: parentNode ? parentNode.key : null
        });
      },
      ignoreCollapsed: false,
      treeData: [{
        key: 1,
        children: [{
          key: 12,
          children: [{ key: 3 }]
        }, { key: 4 }]
      }, { key: 5 }],
      expected: [{
        key: 1,
        parentKey: null,
        children: [{
          key: 12,
          parentKey: 1,
          children: [{
            key: 3,
            parentKey: 12
          }]
        }, {
          key: 4,
          parentKey: 1
        }]
      }, {
        key: 5,
        parentKey: null
      }]
    });
  });

  it('can sort part of the tree', function () {
    [{
      getNodeKey: keyFromKey,
      callback: function callback(_ref20) {
        var node = _ref20.node;
        return !node.children ? node : _extends({}, node, {
          children: node.children.sort(function (a, b) {
            return a.key - b.key;
          })
        });
      },
      treeData: [{
        key: 1,
        expanded: true,
        children: [{
          key: 12,
          expanded: true,
          children: [{ key: 33 }, { key: 3 }]
        }, { key: 4 }]
      }, { key: 5 }],
      expected: [{
        key: 1,
        expanded: true,
        children: [{ key: 4 }, {
          key: 12,
          expanded: true,
          children: [{ key: 3 }, { key: 33 }]
        }]
      }, { key: 5 }]
    }].forEach(checkFunction);
  });

  it('can modify every node in the tree', function () {
    [{
      getNodeKey: keyFromKey,
      callback: function callback(_ref21) {
        var node = _ref21.node;
        return _extends({}, node, { expanded: true });
      },
      ignoreCollapsed: false,
      treeData: [{
        key: 1,
        children: [{
          key: 12,
          children: [{ key: 33 }, { key: 3 }]
        }, { key: 4 }]
      }, { key: 5 }],
      expected: [{
        key: 1,
        expanded: true,
        children: [{
          key: 12,
          expanded: true,
          children: [{ key: 33, expanded: true }, { key: 3, expanded: true }]
        }, { key: 4, expanded: true }]
      }, { key: 5, expanded: true }]
    }].forEach(checkFunction);
  });
});

describe('isDescendant', function () {
  var treeData = [{
    key: 1,
    children: [{
      key: 12,
      children: [{ key: 33 }, { key: 3 }]
    }, { key: 4 }]
  }, { key: 5 }];

  it('should work at the base', function () {
    expect((0, _treeDataUtils.isDescendant)(treeData[0], treeData[0])).toEqual(false);
    expect((0, _treeDataUtils.isDescendant)(treeData[0], treeData[1])).toEqual(false);
    expect((0, _treeDataUtils.isDescendant)(treeData[0], treeData[0].children[1])).toEqual(true);
  });

  it('should work deeper in the tree', function () {
    expect((0, _treeDataUtils.isDescendant)(treeData[0].children[0], treeData[0].children[0].children[1])).toEqual(true);
  });
});

describe('getDepth', function () {
  var treeData = [{
    key: 1,
    children: [{
      key: 12,
      children: [{ key: 33 }, { key: 3 }]
    }, { key: 4 }]
  }, { key: 5 }];

  it('should work at the base', function () {
    expect((0, _treeDataUtils.getDepth)(treeData[0])).toEqual(2);
    expect((0, _treeDataUtils.getDepth)(treeData[1])).toEqual(0);
  });

  it('should work deeper in the tree', function () {
    expect((0, _treeDataUtils.getDepth)(treeData[0].children[0])).toEqual(1);
  });
});

describe('getDescendantCount', function () {
  it('should count flat data', function () {
    expect((0, _treeDataUtils.getDescendantCount)({ ignoreCollapsed: false, node: {} })).toEqual(0);
    expect((0, _treeDataUtils.getDescendantCount)({ ignoreCollapsed: false, node: { children: [] } })).toEqual(0);
    expect((0, _treeDataUtils.getDescendantCount)({ ignoreCollapsed: false, node: { children: [{}] } })).toEqual(1);
    expect((0, _treeDataUtils.getDescendantCount)({
      ignoreCollapsed: false,
      node: { children: [{}, {}] }
    })).toEqual(2);
  });

  it('should count nested data', function () {
    var nested = {
      expanded: true,
      children: [{}, { children: [{}] }, {}]
    };

    expect((0, _treeDataUtils.getDescendantCount)({ ignoreCollapsed: false, node: nested })).toEqual(4);
    expect((0, _treeDataUtils.getDescendantCount)({ ignoreCollapsed: true, node: nested })).toEqual(3);
  });
});

describe('find', function () {
  var commonArgs = {
    searchQuery: 42,
    searchMethod: function searchMethod(_ref22) {
      var node = _ref22.node;
      var searchQuery = _ref22.searchQuery;
      return node.key === searchQuery;
    },
    expandAllMatchPaths: false,
    expandFocusMatchPaths: true,
    getNodeKey: keyFromKey,
    searchFocusOffset: 0
  };

  it('should work with flat data', function () {
    var result = undefined;

    result = (0, _treeDataUtils.find)(_extends({}, commonArgs, { treeData: [{}] }));
    expect(result.matches).toEqual([]);

    result = (0, _treeDataUtils.find)(_extends({}, commonArgs, { treeData: [{ key: 41 }] }));
    expect(result.matches).toEqual([]);

    result = (0, _treeDataUtils.find)(_extends({}, commonArgs, { treeData: [{ key: 42 }] }));
    expect(result.matches).toEqual([{ node: { key: 42 }, treeIndex: 0, path: [42] }]);
    expect(result.matches[commonArgs.searchFocusOffset].treeIndex).toEqual(0);

    result = (0, _treeDataUtils.find)(_extends({}, commonArgs, { treeData: [{ key: 41 }, { key: 42 }] }));
    expect(result.matches).toEqual([{ node: { key: 42 }, treeIndex: 1, path: [42] }]);
    expect(result.matches[commonArgs.searchFocusOffset].treeIndex).toEqual(1);

    result = (0, _treeDataUtils.find)(_extends({}, commonArgs, { treeData: [{ key: 42 }, { key: 42 }] }));
    expect(result.matches).toEqual([{ node: { key: 42 }, treeIndex: 0, path: [42] }, { node: { key: 42 }, treeIndex: 1, path: [42] }]);
    expect(result.matches[commonArgs.searchFocusOffset].treeIndex).toEqual(0);

    result = (0, _treeDataUtils.find)(_extends({}, commonArgs, {
      searchFocusOffset: 3,
      treeData: [{ key: 1 }, { key: 42 }, { key: 3 }, { key: 3 }, { key: 3 }, { key: 4 }, { key: 42 }, { key: 42 }, { key: 4 }, { key: 42 }]
    }));
    expect(result.matches).toEqual([{ node: { key: 42 }, treeIndex: 1, path: [42] }, { node: { key: 42 }, treeIndex: 6, path: [42] }, { node: { key: 42 }, treeIndex: 7, path: [42] }, { node: { key: 42 }, treeIndex: 9, path: [42] }]);
    expect(result.matches[3].treeIndex).toEqual(9);
  });

  it('should work with nested data', function () {
    var result = undefined;

    result = (0, _treeDataUtils.find)(_extends({}, commonArgs, {
      treeData: [{ children: [{ key: 42 }] }]
    }));
    expect(result.matches.length).toEqual(1);
    expect(result.matches[commonArgs.searchFocusOffset].treeIndex).toEqual(1);

    result = (0, _treeDataUtils.find)(_extends({}, commonArgs, {
      treeData: [{ children: [{ key: 41 }] }, { children: [{ key: 42 }] }]
    }));
    expect(result.matches.length).toEqual(1);
    expect(result.matches[commonArgs.searchFocusOffset].treeIndex).toEqual(2);
    expect(result.treeData).toEqual([{ children: [{ key: 41 }] }, { expanded: true, children: [{ key: 42 }] }]);

    result = (0, _treeDataUtils.find)(_extends({}, commonArgs, {
      treeData: [{ children: [{ children: [{ key: 42 }] }] }]
    }));
    expect(result.matches.length).toEqual(1);
    expect(result.matches[commonArgs.searchFocusOffset].treeIndex).toEqual(2);

    result = (0, _treeDataUtils.find)(_extends({}, commonArgs, {
      treeData: [{ children: [{ key: 42, children: [{ key: 42 }] }] }]
    }));
    expect(result.matches.length).toEqual(2);
    expect(result.matches[commonArgs.searchFocusOffset].treeIndex).toEqual(1);

    result = (0, _treeDataUtils.find)(_extends({}, commonArgs, {
      treeData: [{}, { children: [{ key: 42, expanded: true, children: [{ key: 42 }] }] }]
    }));
    expect(result.matches.length).toEqual(2);
    expect(result.matches[commonArgs.searchFocusOffset].treeIndex).toEqual(2);

    result = (0, _treeDataUtils.find)(_extends({}, commonArgs, {
      treeData: [{}, { children: [{ key: 1, expanded: true, children: [{ key: 1 }] }] }]
    }));
    expect(result.matches.length).toEqual(0);
  });
});